from __future__ import generators
import plus
import Arenas
import Hazards
import random

import math

class BoxArena(Arenas.SuperArena):
    "Located in a vault far beneath the busy city streets lies what seasoned bot builders call the Trauma Pit. Well known for its vile and aggressive competition, those who fight here take no prisoners."
    name = "The Trauma Pit"
    preview = "TraumaPit/traumapit_preview.bmp"
    game_types = ['DEATHMATCH', 'BATTLE ROYAL', 'TEAM MATCH']

    def __init__(self):
        Arenas.SuperArena.__init__(self, "Arenas/TraumaPit/traumapit.gmf")
        self.players = ()
        plus.setBackColor(0, 0, 0)
        
        fmod = .015
        # static cameras
        #self.AddStaticCamera("Static 1", (-3.5, 9, -7.7), (math.pi / 6.0, 0), math.pi / 4.0)
        self.AddStaticCamera("Static 1", (0, 10, -28), (.45, 0), fmod*50)
        self.AddStaticCamera("Static 2", (0, 48, 0), (math.pi/2.01, 0), fmod*37)
        self.AddStaticCamera("Static 3", (19.7554,6.5743,-25.141), (0.266024,-0.672956), fmod*52)
        
        # watch camera
        #self.AddWatchCamera("Watch 1", (-12.1, 6.8, -11.7), (2.0, 20.0, math.pi / 3.5, math.pi / 9.0))
        self.AddWatchCamera("Watch 1", (-12.1, 0.1, -11.7), (2.0, 30.0, fmod*60, fmod*25))
        self.AddWatchCamera("Watch 2", (1.7, 0, -17.4), (2.0, 30.0, fmod*60, fmod*25))
        self.AddWatchCamera("Watch 3", (-12.1, 7.0, -11.7), (2.0, 30.0, fmod*60, fmod*25))
		
		
        
        #Pre-cache Battle Music
        self.battle_music = plus.createSound("Sounds/intro_music/embattled.wav", False, (0,0,0))

    def AddShadowReceivers(self):
        #Shadow Light Sources
        self.SetShadowSource(0.653, 6.894, -0.418)
        
        #Main Arena Shadow Triangles
        self.AddShadowTriangle((-12.7,-2.1126,13.0027), (-12.7,-2.1126,-12.3973), (12.7,-2.1126,13.0027))
        self.AddShadowTriangle((12.7,-2.1126,-12.3973), (12.7,-2.1126,13.0027), (-12.7,-2.1126,-12.3973))
        self.AddShadowTriangle((12.7,-2.1126,-12.3973), (-12.7,-2.1126,-12.3973), (12.7,2.2245,-12.3973))
        self.AddShadowTriangle((-12.7,2.2245,-12.3973), (12.7,2.2245,-12.3973), (-12.7,-2.1126,-12.3973))
        self.AddShadowTriangle((12.7,-2.1126,13.0027), (12.7,-2.1126,-12.3973), (12.7,2.2245,13.0027))
        self.AddShadowTriangle((12.7,2.2245,-12.3973), (12.7,2.2245,13.0027), (12.7,-2.1126,-12.3973))
        self.AddShadowTriangle((-12.7,-2.1126,13.0027), (12.7,-2.1126,13.0027), (-12.7,2.2245,13.0027))
        self.AddShadowTriangle((12.7,2.2245,13.0027), (-12.7,2.2245,13.0027), (12.7,-2.1126,13.0027))
        self.AddShadowTriangle((-12.7,-2.1126,-12.3973), (-12.7,-2.1126,13.0027), (-12.7,2.2245,-12.3973))
        self.AddShadowTriangle((-12.7,2.2245,13.0027), (-12.7,2.2245,-12.3973), (-12.7,-2.1126,13.0027))
        
    def HazardsOn(self, on):
        if on:
            prism = self.AddPrismatic("arena_collision", "spikes1", 0, 1, 0, 0, .8, 0)
            self.spikes1 = Hazards.Spikes(prism, 40000, (-300.7, -2.2, -5.2))
            prism = self.AddPrismatic("arena_collision", "spikes2", 0, 1, 0, 0, .8, 0)
            self.spikes2 = Hazards.Spikes(prism, 40000, (400.9, -2.2, -5.2))
    
            prism = self.AddPrismatic("arena_collision", "floorsaw", 0, 1, 0, -.1, .46, 0)
            self.saw = Hazards.Saws(prism, (-.1, -2.2, 500.4))
    
            self.AddHazard(self.spikes1)
            self.AddHazard(self.spikes2)
            self.AddHazard(self.saw)
            
            # hazards
            self.AddCollisionLine((-1.6, 6.7), (1.3, 6.7))
            self.AddCollisionLine((1.3, 6.7), (1.3, 4))
            self.AddCollisionLine((1.3, 4), (-1.6, 4))
            self.AddCollisionLine((-1.6, 4), (-1.6, 6.7))
            
            self.AddCollisionLine((-4.4, -3.9), (-3, -3.9))
            self.AddCollisionLine((-3, -3.9), (-3, -6.3))
            self.AddCollisionLine((-3, -6.3), (-4.4, -6.3))
            self.AddCollisionLine((-4.4, -6.3), (-4.4, -3.9))
            
            self.AddCollisionLine((4.2, -3.9), (5.6, -3.9))
            self.AddCollisionLine((5.6, -3.9), (5.6, -6.4))
            self.AddCollisionLine((5.6, -6.4), (4.2, -6.4))
            self.AddCollisionLine((4.2, -6.4), (4.2, -3.9))

            self.AddPOV(0, (0, 10), (1, 3))
            self.AddPOV(1, (5, 5.5), (0, 2, 8))
            self.AddPOV(2, (0, 1), (1, 3, 4, 8, 5, 11))
            self.AddPOV(3, (-5, 5.5), (0, 2, 4))
            self.AddPOV(4, (-4, -1), (2, 3, 5, 7))
            self.AddPOV(5, (0, -5.3), (2, 4, 6, 11))
            self.AddPOV(6, (-4, -9), (5, 7, 10))
            self.AddPOV(7, (-7, -5.3), (4, 6))
            self.AddPOV(8, (5, -1), (1, 2, 9, 11))
            self.AddPOV(9, (8, -5.3), (8, 10))
            self.AddPOV(10, (5, -9), (6, 9, 11))
            self.AddPOV(11, (1.5, -5.3), (2, 8, 5, 10))
 
        else:
            # hazards should be in their "hidden" position
            pass
            
        # walls
        self.AddCollisionLine((-12.7, 13), (-12.7, -12.4))
        self.AddCollisionLine((-12.7, -12.4), (12.7, -12.4))
        self.AddCollisionLine((12.7, -12.4), (12.7, 13))
        self.AddCollisionLine((12.7, 13), (-12.7, 13))

        return Arenas.SuperArena.HazardsOn(self, on)

    def Activate(self, on):
        if on:
            self.players = plus.getPlayers()
        
        Arenas.SuperArena.Activate(self, on)
        
    def Introduction(self):
        sounds = self.intro_sounds
        
        # set initial camera & fade from black
        plus.setCameraPosition(19.7554,6.5743,-25.141)
        plus.setCameraRotation(0.266024,-0.472956)
        plus.setCameraFOV(0.9)
        plus.fadeFromBlack(.25)

        #start playing music loop
        self.intro_music = plus.createSound("Sounds/intro_music/embattled.wav", False, (0,0,0))
        plus.setVolume(self.intro_music, 0, 0)
        plus.loopSound(self.intro_music)
        yield .25
        
        #load all sounds now to decrease lag later
        sounds['crowd'] = plus.createSound("Sounds/announcers/silence.wav", False, (0,0,0))
        
        arenaOpt = ("Sounds/announcers/silence.wav")
        sounds['arena'] = plus.createSound(random.choice(arenaOpt), False, (0,0,0))
        genericOpt = ("silence.wav")
        sounds['generic'] = plus.createSound("Sounds/announcers/"+random.choice(genericOpt), False, (0,0,0))
        hazardOpt = ("Sounds/announcers/silence.wav")
        sounds['hazards'] = plus.createSound(random.choice(hazardOpt), False, (0,0,0))
        botOpt = ("silence.wav")
        sounds['bots'] = plus.createSound("Sounds/announcers/"+random.choice(botOpt), False, (0,0,0))
        
        #intro cam, welcom comment
        plus.playSound(sounds['arena'])
        plus.animateCamera((19.7554,6.5743,-25.141), (0.266024,-0.472956), 0.9, (20.3992,4.28651,8.97787), (0.193915,-1.95089), 0.9, 0, 8)
        yield 2
        plus.fadeOutLoop(sounds['crowd'], 8000)
        yield 1
        
        #play a generic (or specific) secondary comment
        plus.playSound(sounds['generic'])
        yield 5
        
        #hazard cams
        if self.bHazardsOn:
            plus.playSound(sounds['hazards'])
            plus.animateCamera((-0.6,3.0,-15.2), (0.3,-0.3), 0.9, (-0.6,3.0,-15.6), (0.4,0.65), 0.9, 0, 8)
            yield 4

        players = plus.getPlayers()
        pcount = len(players)
        if pcount>0: plus.playSound(sounds['bots'])
        delaytime = 6 - pcount
        
        if 0 in players:
            #bot 1 cam
            plus.animateCamera((9.06563,0.12132,-3.08764), (0.210442,-0.141627), 0.8862, (9.39987,0.12132,-2.25815), (0.494558,-0.12997), 0.8862, 0, delaytime)
            yield delaytime
            
        if 1 in players:
            #bot 2 cam
            plus.animateCamera((-12.22435,0.09861,2.95753), (0.47197,2.53793), 0.8862, (-12.59523,-0.911434,2.42526), (0.147264,2.22609), 0.8862, 0, delaytime)
            yield delaytime
            
        if 2 in players:
            #bot 3 cam
            plus.animateCamera((-0.06,0.12,5.58), (0.11,-0.14), 0.88, (0.36,0.12,5.58), (0.29,-0.12), 0.88, 0, delaytime)
            yield delaytime
            
        if 3 in players:
            #bot 4 cam
            plus.animateCamera((-3.06,0.12,-5.58), (0.39,2.50), 0.88, (-3.06,0.12,-7.58), (0.39,2.00), 0.88, 0, delaytime)
            yield delaytime
            
        #fade out music
        plus.fadeOutLoop(self.intro_music, 2000)
        yield 2
        
        # done
        yield 0
        
        
    def Tick(self):
        for player in self.players:
            if plus.getLocation(player)[1] < -5:
                plus.eliminatePlayer(player)
        return Arenas.SuperArena.Tick(self)

    def Winner(self, id1, id2):
        #Cut Music
        if self.battle_music:
                plus.stopSound(self.battle_music)
                plus.removeSound(self.battle_music)
                
        #print "Winners:", id1, id2
        return Arenas.SuperArena.Winner(self, id1, id2)


Arenas.register(BoxArena)





